package sun.security.x509;

import java.util.Enumeration;
import sun.security.util.DerOutputStream;
import java.io.OutputStream;
import sun.security.util.DerInputStream;
import sun.security.util.DerValue;
import java.io.IOException;

public class IssuingDistributionPointExtension extends Extension implements CertAttrSet<String>
{
    public static final String IDENT = "x509.info.extensions.IssuingDistributionPoint";
    public static final String NAME = "IssuingDistributionPoint";
    public static final String POINT = "point";
    public static final String REASONS = "reasons";
    public static final String ONLY_USER_CERTS = "only_user_certs";
    public static final String ONLY_CA_CERTS = "only_ca_certs";
    public static final String ONLY_ATTRIBUTE_CERTS = "only_attribute_certs";
    public static final String INDIRECT_CRL = "indirect_crl";
    private DistributionPointName distributionPoint;
    private ReasonFlags revocationReasons;
    private boolean hasOnlyUserCerts;
    private boolean hasOnlyCACerts;
    private boolean hasOnlyAttributeCerts;
    private boolean isIndirectCRL;
    private static final byte TAG_DISTRIBUTION_POINT = 0;
    private static final byte TAG_ONLY_USER_CERTS = 1;
    private static final byte TAG_ONLY_CA_CERTS = 2;
    private static final byte TAG_ONLY_SOME_REASONS = 3;
    private static final byte TAG_INDIRECT_CRL = 4;
    private static final byte TAG_ONLY_ATTRIBUTE_CERTS = 5;
    
    public IssuingDistributionPointExtension(DistributionPointName distributionPoint, ReasonFlags revocationReasons, boolean hasOnlyUserCerts, boolean hasOnlyCACerts, boolean hasOnlyAttributeCerts, boolean isIndirectCRL) throws IOException {
        this.distributionPoint = null;
        this.revocationReasons = null;
        this.hasOnlyUserCerts = false;
        this.hasOnlyCACerts = false;
        this.hasOnlyAttributeCerts = false;
        this.isIndirectCRL = false;
        if ((hasOnlyUserCerts && (hasOnlyCACerts || hasOnlyAttributeCerts)) || (hasOnlyCACerts && (hasOnlyUserCerts || hasOnlyAttributeCerts)) || (hasOnlyAttributeCerts && (hasOnlyUserCerts || hasOnlyCACerts))) {
            throw new IllegalArgumentException("Only one of hasOnlyUserCerts, hasOnlyCACerts, hasOnlyAttributeCerts may be set to true");
        }
        this.extensionId = PKIXExtensions.IssuingDistributionPoint_Id;
        this.critical = true;
        this.distributionPoint = distributionPoint;
        this.revocationReasons = revocationReasons;
        this.hasOnlyUserCerts = hasOnlyUserCerts;
        this.hasOnlyCACerts = hasOnlyCACerts;
        this.hasOnlyAttributeCerts = hasOnlyAttributeCerts;
        this.isIndirectCRL = isIndirectCRL;
        this.encodeThis();
        return;
    }
    
    public IssuingDistributionPointExtension(Boolean b, Object o) throws IOException {
        this.distributionPoint = null;
        this.revocationReasons = null;
        this.hasOnlyUserCerts = false;
        this.hasOnlyCACerts = false;
        this.hasOnlyAttributeCerts = false;
        this.isIndirectCRL = false;
        this.extensionId = PKIXExtensions.IssuingDistributionPoint_Id;
        this.critical = (boolean)b;
        if (!(o instanceof byte[])) {
            throw new IOException("Illegal argument type");
        }
        this.extensionValue = (byte[])(byte[])o;
        DerValue derValue = new DerValue(this.extensionValue);
        if (derValue.tag != 48) {
            throw new IOException("Invalid encoding for IssuingDistributionPointExtension.");
        }
        if (derValue.data == null || derValue.data.available() == 0) {
            return;
        }
        DerInputStream data = derValue.data;
        while (data != null && data.available() != 0) {
            DerValue derValue2 = data.getDerValue();
            if (derValue2.isContextSpecific((byte)0) && derValue2.isConstructed()) {
                this.distributionPoint = new DistributionPointName(derValue2.data.getDerValue());
            }
            else if (derValue2.isContextSpecific((byte)1) && !derValue2.isConstructed()) {
                derValue2.resetTag((byte)1);
                this.hasOnlyUserCerts = derValue2.getBoolean();
            }
            else if (derValue2.isContextSpecific((byte)2) && !derValue2.isConstructed()) {
                derValue2.resetTag((byte)1);
                this.hasOnlyCACerts = derValue2.getBoolean();
            }
            else if (derValue2.isContextSpecific((byte)3) && !derValue2.isConstructed()) {
                this.revocationReasons = new ReasonFlags(derValue2);
            }
            else if (derValue2.isContextSpecific((byte)4) && !derValue2.isConstructed()) {
                derValue2.resetTag((byte)1);
                this.isIndirectCRL = derValue2.getBoolean();
            }
            else if (derValue2.isContextSpecific((byte)5) && !derValue2.isConstructed()) {
                derValue2.resetTag((byte)1);
                this.hasOnlyAttributeCerts = derValue2.getBoolean();
            }
            else {
                throw new IOException("Invalid encoding of IssuingDistributionPoint");
            }
        }
        return;
    }
    
    @Override
    public String getName() {
        return "IssuingDistributionPoint";
    }
    
    @Override
    public void encode(OutputStream outputStream) throws IOException {
        DerOutputStream derOutputStream = new DerOutputStream();
        if (this.extensionValue == null) {
            this.extensionId = PKIXExtensions.IssuingDistributionPoint_Id;
            this.critical = false;
            this.encodeThis();
        }
        super.encode(derOutputStream);
        outputStream.write(derOutputStream.toByteArray());
    }
    
    @Override
    public void set(String str, Object o) throws IOException {
        if (str.equalsIgnoreCase("point")) {
            if (!(o instanceof DistributionPointName)) {
                throw new IOException("Attribute value should be of type DistributionPointName.");
            }
            this.distributionPoint = (DistributionPointName)o;
        }
        else if (str.equalsIgnoreCase("reasons")) {
            if (!(o instanceof ReasonFlags)) {
                throw new IOException("Attribute value should be of type ReasonFlags.");
            }
            this.revocationReasons = (ReasonFlags)o;
        }
        else if (str.equalsIgnoreCase("indirect_crl")) {
            if (!(o instanceof Boolean)) {
                throw new IOException("Attribute value should be of type Boolean.");
            }
            this.isIndirectCRL = (boolean)o;
        }
        else if (str.equalsIgnoreCase("only_user_certs")) {
            if (!(o instanceof Boolean)) {
                throw new IOException("Attribute value should be of type Boolean.");
            }
            this.hasOnlyUserCerts = (boolean)o;
        }
        else if (str.equalsIgnoreCase("only_ca_certs")) {
            if (!(o instanceof Boolean)) {
                throw new IOException("Attribute value should be of type Boolean.");
            }
            this.hasOnlyCACerts = (boolean)o;
        }
        else if (str.equalsIgnoreCase("only_attribute_certs")) {
            if (!(o instanceof Boolean)) {
                throw new IOException("Attribute value should be of type Boolean.");
            }
            this.hasOnlyAttributeCerts = (boolean)o;
        }
        else {
            throw new IOException("Attribute name [" + str + "] not recognized by CertAttrSet:IssuingDistributionPointExtension.");
        }
        this.encodeThis();
    }
    
    @Override
    public Object get(String str) throws IOException {
        if (str.equalsIgnoreCase("point")) {
            return this.distributionPoint;
        }
        if (str.equalsIgnoreCase("indirect_crl")) {
            return Boolean.valueOf(this.isIndirectCRL);
        }
        if (str.equalsIgnoreCase("reasons")) {
            return this.revocationReasons;
        }
        if (str.equalsIgnoreCase("only_user_certs")) {
            return Boolean.valueOf(this.hasOnlyUserCerts);
        }
        if (str.equalsIgnoreCase("only_ca_certs")) {
            return Boolean.valueOf(this.hasOnlyCACerts);
        }
        if (str.equalsIgnoreCase("only_attribute_certs")) {
            return Boolean.valueOf(this.hasOnlyAttributeCerts);
        }
        throw new IOException("Attribute name [" + str + "] not recognized by CertAttrSet:IssuingDistributionPointExtension.");
    }
    
    @Override
    public void delete(String str) throws IOException {
        if (str.equalsIgnoreCase("point")) {
            this.distributionPoint = null;
        }
        else if (str.equalsIgnoreCase("indirect_crl")) {
            this.isIndirectCRL = false;
        }
        else if (str.equalsIgnoreCase("reasons")) {
            this.revocationReasons = null;
        }
        else if (str.equalsIgnoreCase("only_user_certs")) {
            this.hasOnlyUserCerts = false;
        }
        else if (str.equalsIgnoreCase("only_ca_certs")) {
            this.hasOnlyCACerts = false;
        }
        else if (str.equalsIgnoreCase("only_attribute_certs")) {
            this.hasOnlyAttributeCerts = false;
        }
        else {
            throw new IOException("Attribute name [" + str + "] not recognized by CertAttrSet:IssuingDistributionPointExtension.");
        }
        this.encodeThis();
    }
    
    @Override
    public Enumeration<String> getElements() {
        AttributeNameEnumeration attributeNameEnumeration = new AttributeNameEnumeration();
        attributeNameEnumeration.addElement("point");
        attributeNameEnumeration.addElement("reasons");
        attributeNameEnumeration.addElement("only_user_certs");
        attributeNameEnumeration.addElement("only_ca_certs");
        attributeNameEnumeration.addElement("only_attribute_certs");
        attributeNameEnumeration.addElement("indirect_crl");
        return attributeNameEnumeration.elements();
    }
    
    private void encodeThis() throws IOException {
        if (this.distributionPoint == null && this.revocationReasons == null && !this.hasOnlyUserCerts && !this.hasOnlyCACerts && !this.hasOnlyAttributeCerts && !this.isIndirectCRL) {
            this.extensionValue = null;
            return;
        }
        DerOutputStream derOutputStream = new DerOutputStream();
        if (this.distributionPoint != null) {
            DerOutputStream derOutputStream2 = new DerOutputStream();
            this.distributionPoint.encode(derOutputStream2);
            derOutputStream.writeImplicit(DerValue.createTag((byte)(-128), true, (byte)0), derOutputStream2);
        }
        if (this.hasOnlyUserCerts) {
            DerOutputStream o = new DerOutputStream();
            o.putBoolean(this.hasOnlyUserCerts);
            derOutputStream.writeImplicit(DerValue.createTag((byte)(-128), false, (byte)1), o);
        }
        if (this.hasOnlyCACerts) {
            DerOutputStream o2 = new DerOutputStream();
            o2.putBoolean(this.hasOnlyCACerts);
            derOutputStream.writeImplicit(DerValue.createTag((byte)(-128), false, (byte)2), o2);
        }
        if (this.revocationReasons != null) {
            DerOutputStream o3 = new DerOutputStream();
            this.revocationReasons.encode(o3);
            derOutputStream.writeImplicit(DerValue.createTag((byte)(-128), false, (byte)3), o3);
        }
        if (this.isIndirectCRL) {
            DerOutputStream o4 = new DerOutputStream();
            o4.putBoolean(this.isIndirectCRL);
            derOutputStream.writeImplicit(DerValue.createTag((byte)(-128), false, (byte)4), o4);
        }
        if (this.hasOnlyAttributeCerts) {
            DerOutputStream o5 = new DerOutputStream();
            o5.putBoolean(this.hasOnlyAttributeCerts);
            derOutputStream.writeImplicit(DerValue.createTag((byte)(-128), false, (byte)5), o5);
        }
        DerOutputStream o6 = new DerOutputStream();
        o6.write((byte)48, derOutputStream);
        this.extensionValue = o6.toByteArray();
        return;
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder(super.toString());
        sb.append("IssuingDistributionPoint [\n  ");
        if (this.distributionPoint != null) {
            sb.append(this.distributionPoint);
        }
        if (this.revocationReasons != null) {
            sb.append(this.revocationReasons);
        }
        sb.append(this.hasOnlyUserCerts ? "  Only contains user certs: true" : "  Only contains user certs: false").append("\n");
        sb.append(this.hasOnlyCACerts ? "  Only contains CA certs: true" : "  Only contains CA certs: false").append("\n");
        sb.append(this.hasOnlyAttributeCerts ? "  Only contains attribute certs: true" : "  Only contains attribute certs: false").append("\n");
        sb.append(this.isIndirectCRL ? "  Indirect CRL: true" : "  Indirect CRL: false").append("\n");
        sb.append("]\n");
        return sb.toString();
    }
}
